/*
===============================================================================
 Name        : wifly.c
 Author      : Eduardo Rafael Sepulveda de Ugarte
 Version     : 1.0
 Description : Wifly header file

 ===============================================================================
 */

#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "task.h"
#include "semphr.h"

#include "lpc_types.h"
#include "printf.h"
#include "wifly.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>




static char Buffer[GetSIZE];
static uint32_t Estat = NORMAL, Error_Send = 0, Error_Read = 0;
//static uint32_t Error_Send = 0;
//static uint32_t Error_Read = 0;

/* Declare a variable of type xSemaphoreHandle */
volatile static xSemaphoreHandle wMutex = NULL;

/*****************************************************************************
 ** Function name:		Lock_Wifly
 **
 ** Descriptions:		Use Semaphore or vTaskSuspend for block process
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void Lock_Wifly() {
	if (wMutex == NULL ) {
		wMutex = xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(wMutex, portMAX_DELAY);
}

/*****************************************************************************
 ** Function name:		UnLock_Wifly
 **
 ** Descriptions:		Use Semaphore or vTaskResume for unblock process
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void UnLock_Wifly() {
	xSemaphoreGiveRecursive(wMutex);
}

/*****************************************************************************
 ** Function name:		Wifly_Reset
 **
 ** Descriptions:		Hardware reset the WiFly device. PIN 6-24 (P0.22) of LPC1769
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void Wifly_Reset() {

	// Initialize pin
	LPC_PINCON ->PINSEL2 &= (~(3 << 14));
	LPC_GPIO0 ->FIODIR |= (1 << WIFLY_PIN_RST);

	// it made reset at Wifly
	LPC_GPIO0 ->FIOCLR = 1 << WIFLY_PIN_RST;
	Delay_Ms(DELAY, 1); // min 160us
	LPC_GPIO0 ->FIOSET = 1 << WIFLY_PIN_RST;
}

/*****************************************************************************
 ** Function name:		Wifly_Init
 **
 ** Descriptions:		Defines the used UART, its baudrate and reset device
 ** 						This operation produce a few delay.
 **
 ** parameters:			UART port ,  desired baudrate
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Init(uint8_t PortNum, uint32_t Baudrate) {
	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		Print_Flush(PortNum);
		ret = Print_Init(PortNum, Baudrate);
		//Wifly_Reset();//set correct i/o pin
		//Delay_Ms(DELAY, 1000);  // min 160us
		Print_Flush(PortNum);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_End
 **
 ** Descriptions:		Close the UART port used for WiFly device.
 **
 ** parameters:			UART port
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_End(uint8_t PortNum) {
	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		Print_Flush(PortNum);
		ret = Print_End(PortNum);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set(uint8_t PortNum, char *Comand, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';

	printf( "SENDING: %s\n",Comand );
	Print_Text(UART_DBG,Comand);

	Print_T_CRLF(PortNum, Comand);
	if (UART_Read(PortNum, (char *) Buffer, (char *) "\n", Time) > 0)
	{
		ret = TRUE;
		printf( "Wifly_Set READED: %s\n",Buffer );
		Print_Text(UART_DBG,Buffer);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_OK(uint8_t PortNum, char *Comand, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer to join at command
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TT_OK(uint8_t PortNum, char *Comand, char *Value,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TN_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, number to join at command
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TN_OK(uint8_t PortNum, char *Comand, int32_t Num,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TN_CRLF(PortNum, Comand, Num);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_Check
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer to check
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_Check(uint8_t PortNum, char *Comand, char *Check,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *) Buffer, Check, Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_Check
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer Value to join at command
 ** 						char pointer to Check, and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TT_Check(uint8_t PortNum, char *Comand, char *Value,
		char *Check, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
		Print_Flush(PortNum);
		//Buffer[0] = '\0';
		Print_TT_CRLF(PortNum, Comand, Value);
		ret = Check_Read(PortNum, (char *) Buffer, Check, Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_CMD
 **
 ** Descriptions:		Send '$$$' to enter at Command Mode of WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_CMD(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	/*
	if (Estat == COMMAND){
		printf("Ya estaba en modo comando!!!!\n");
		return TRUE;
	}
    */
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_Text(PortNum, (char *) "$$$");
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "CMD", 300);
	if (ret == TRUE)
		Estat = COMMAND;
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_DATA
 **
 ** Descriptions:		Send '$$$' to enter at Command Mode of WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_DATA(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	if (Estat == NORMAL)
		return TRUE;
	Print_Text(PortNum, (char *) "exit");
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "EXIT", 300);
	if (ret == TRUE)
		Estat = NORMAL;
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Connect
 **
 ** Descriptions:		Connect to a Wi-Fy network
 **
 ** parameters:			Pointer to Buffer and Time to wait in milliseconds
 ** Returned value:		Number of characters read
 **
 *****************************************************************************/
uint32_t Wifly_Connect(uint8_t PortNum, uint32_t Mode, char *ssid, char* Phrase) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
    Wifly_Set_TN_OK(PortNum, (char *) "set w a", Mode, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w s", ssid, 5000);	//500
	Wifly_Set_TT_OK(PortNum, (char *) "set w p", Phrase, 5000);	//750
	ret = Wifly_Set_TT_Check(PortNum, (char *) "join", ssid,
			(char *) "Associated!", 5000);
	UnLock_Wifly();
	return ret;
}



/*****************************************************************************
 ** Function name:		Wifly_Disconnect
 **
 ** Descriptions:		Send 'leave' to disconnect the WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/

uint32_t Wifly_Disconnect(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char*) "leave", (char *) "DeAuth", 500);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Config_UDP
 **
 ** Descriptions:		Setting configuration to allow Wifly work with UDP
 **
 **
 ** parameters:
 ** Returned value:
 **
 *****************************************************************************/
uint32_t Wifly_Config_UDP() {

	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		printf( "\nConfig WiFly for UDP..." );

		Wifly_Set (UART_WF,"set ip proto 1",500);					//Enable UDP

		Wifly_Set (UART_WF,"set broadcast interval 0",500);			//Disable Broadcast

		Wifly_Set (UART_WF,"set ip host 255.255.255.255",500);		//255.255.255.255
		Wifly_Set (UART_WF,"set ip flags 0x00",500);				//Disable Autoparing

		Wifly_Set (UART_WF,"set ip remote 8888",500);
		Wifly_Set (UART_WF,"set ip local 8888",500);
		Wifly_Set (UART_WF,"set wlan join 1",500);

		//Direct communication TEST
		//Wifly_Set (UART_WF,"set ip host 192.168.1.38",500);

		//For Autoparing mode
		//Wifly_Set (UART_WF,"set ip host 0.0.0.0",500);
		//Wifly_Set (UART_WF,"set ip flags 0x40",500);

		//For Broadcast
		//Wifly_Set (UART_WF,"set broadcast address 255.255.255.255",500);
		//Wifly_Set (UART_WF,"set broadcast interval 2",500);
		//Wifly_Set (UART_WF,"set broadcast port 8888",500);

	}
	UnLock_Wifly();

	printf( "\nSaving WiFly UDP configuration..." );
	Wifly_Set (UART_WF,"save",500);

	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Ping
 **
 ** Descriptions:		Send 'ping' with Address
 **
 ** parameters:			UART port to send and Address to check ping
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Ping(uint8_t PortNum, char *Address) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	ret = Wifly_Set_TT_Check(PortNum, (char *) "ping", Address, (char *) "PING",
			2000);
#ifdef MODE_DEBUG
	if (ret == TRUE) {
		char *string;
		string = strstr(Buffer, (char *) "PING");
		string = strtok(string, (char *) "\r\n");
		Print_TT_CRLF(UART_DBG, (char *) "Resultado de Ping es :", string);
	}
#endif
	UnLock_Wifly();
	return ret;
}






